@@ -9,18 +9,19 @@ CREATE TABLE url (
   status INT,
   url CHAR(128) NOT NULL,
   content_type CHAR(32),
-  last_modified CHAR(48),
   title CHAR(128),
   txt CHAR(255),
   docsize INT,
   last_index_time INT NOT NULL,
   next_index_time INT NOT NULL,
+  last_mod_time INT NOT NULL,
   referrer INT,
-  tag INT,
+  tag CHAR(10),
   hops INT,
+  category CHAR(10),
   keywords CHAR(255),
   description CHAR(100),
-  crc CHAR(33) NOT NULL,
+  crc32 INT NOT NULL,
   lang CHAR(2)
 );
 
@@ -40,7 +41,7 @@ CREATE   INDEX dict_url_id ON dict (url_id);
 
 CREATE UNIQUE  INDEX url_url ON url (url);
 
-CREATE INDEX url_crc ON url (crc);
+CREATE INDEX url_crc ON url (crc32);
 
 CREATE UNIQUE  INDEX stopword_word ON stopword (word,lang);
 
