@@ -29,6 +29,8 @@ extern __INDLIB__ char* UdmConfErrMsg();
 extern __INDLIB__ int	UdmSetDBAddr(char * str);
 extern __INDLIB__ int	UdmSetDBMode(char * str);
 extern __INDLIB__ int   UdmSetDBType(char *str);
+extern __INDLIB__ int   UdmSetTrackMode(int mode);
+
 
 /* Database parameters */
 extern char *DBHost;		/* EXT */
@@ -40,6 +42,7 @@ extern int  DBType;		/* EXT */
 extern int  DBMode;		/* EXT */
 extern int  DBUseLock;		/* EXT */
 extern int Force1251;		/* EXT */
+extern int TrackMode;		/* EXT */
 
 
 #endif
