@@ -6,6 +6,10 @@
 #include <syslog.h>
 #endif
 
+/* According to some recommendations, try not to exceed about 800 bytes 
+   or you might have problems with old syslog implementations */
+#define UDM_LOG_BUF_LEN 255
+
 /* Verbose levels */
 #define UDM_LOG_NONE	0
 #define UDM_LOG_ERROR	1
