@@ -14,7 +14,9 @@
 #else
 #define __INDLIB__
 #define UDMSLASH		'/'
+#include <unistd.h> /* for sleep prototype */
 #define UDMSLEEP(x)		sleep(x)
+#include <string.h> /* for bzero prototype */
 #define UDMMEMZERO(a,b)		bzero(a,b)
 #endif
 
