@@ -19,10 +19,12 @@
 #include "udm_hrefs.h"
 #include "udm_robots.h"
 #include "udm_utils.h"
+#include "udm_xmalloc.h"
 
 int UdmInit(){
 	UdmInitCharset();
 	UdmInitDB();
+	UdmInitTZ();
 	return(0);
 }
 
@@ -35,6 +37,7 @@ int  DBMode	   = UDM_DBMODE_SINGLE;
 int  DBPort        = 0;
 int  DBUseLock     = 1;
 int  Force1251     = 0;
+int  TrackMode     = 0;
 
 typedef struct filter_struct {
 	regex_t filter;
@@ -47,7 +50,6 @@ static char	_extra_headers[UDMSTRSIZ]="";
 static char	_user_agent[UDMSTRSIZ]="";
 static int	_max_doc_size=UDM_MAXDOCSIZE;
 
-
 char * UdmExtraHeaders(){
 	return(_extra_headers);
 }
@@ -227,6 +229,10 @@ static int AddServer(UDM_SERVER * srv){
 	Server[nservers].basic_auth=srv->basic_auth?strdup(srv->basic_auth):NULL;
 	Server[nservers].htdb_list=srv->htdb_list?strdup(srv->htdb_list):NULL;
 	Server[nservers].htdb_doc=srv->htdb_doc?strdup(srv->htdb_doc):NULL;
+        Server[nservers].user=srv->user?strdup(srv->user):NULL;
+        Server[nservers].passwd=srv->passwd?strdup(srv->passwd):NULL;
+        Server[nservers].category=srv->category?strdup(srv->category):NULL;
+        Server[nservers].tag=srv->tag?strdup(srv->tag):NULL;
 	if(srv->proxy){
 		char *s;
 		Server[nservers].proxy=strdup(srv->proxy);
@@ -241,7 +247,6 @@ static int AddServer(UDM_SERVER * srv){
 		Server[nservers].proxy_port=0;
 	}
 	Server[nservers].period=srv->period;
-	Server[nservers].hint=srv->hint;
 	Server[nservers].net_errors=0;
 	Server[nservers].outside=srv->outside;
 	Server[nservers].maxhops=srv->maxhops;
@@ -280,6 +285,8 @@ int i;
 		UDM_FREE(Server[i].htdb_list);
 		UDM_FREE(Server[i].htdb_doc);
 		UDM_FREE(Server[i].proxy);
+		UDM_FREE(Server[i].category);
+		UDM_FREE(Server[i].tag);
 	}
 	nservers=mservers=0;
 	UDM_FREE(Server);
@@ -318,9 +325,10 @@ static int InitServer(UDM_SERVER * srv){
 	srv->charset=NULL;
 	srv->htdb_list=NULL;
 	srv->htdb_doc=NULL;
+	srv->category=NULL;
 	srv->proxy_port=0;
 	srv->period=UDM_DEFAULT_REINDEX_TIME;
-	srv->hint=0;
+	srv->tag=NULL;
 	srv->net_errors=0;
 	srv->max_net_errors=UDM_MAXNETERRORS;
 	srv->read_timeout=UDM_READ_TIMEOUT;
@@ -373,6 +381,10 @@ int m;
 	if(!UDM_STRNCASECMP(str1,"word2url"))m=UDM_DBMODE_WORD2URL;
 	return(m);
 }
+__INDLIB__ int UdmSetTrackMode(int mode){
+	TrackMode=mode;
+	return(TrackMode);
+}
 __INDLIB__ int UdmSetDBMode(char *str){
 	DBMode=UdmStr2DBMode(str);
 	return(DBMode);
@@ -390,6 +402,8 @@ int t=UDM_DB_UNK;
 	if(!UDM_STRNCASECMP(str1,"mysql"))t=UDM_DB_MYSQL;
 	else
 	if(!UDM_STRNCASECMP(str1,"pgsql"))t=UDM_DB_PGSQL;
+	else
+	if(!UDM_STRNCASECMP(str1,"ibase"))t=UDM_DB_IBASE;
 	return(t);
 }
 __INDLIB__ int UdmSetDBType(char *str){
@@ -400,12 +414,24 @@ __INDLIB__ int UdmSetDBAddr(char * str){
 UDM_URL url;
 char *s;
 int res;
+
 	if((res=UdmParseURL(&url,str)))
 		return(res);
 
 	DBHost=strdup(url.hostname);
-	DBName=strdup(url.path);
-	sscanf(url.path,"/%[^/]s",DBName);
+	DBType=UdmStr2DBType(url.schema);
+
+	if(DBType==UDM_DB_IBASE){
+		/* Ibase is a special case        */
+		/* It's database name consists of */
+		/* full path and file name        */ 
+		DBName=malloc(strlen(url.path)+1+strlen(url.filename));
+		sprintf(DBName,"%s%s",url.path,url.filename);
+	}else{
+		DBName=strdup(url.path);
+		sscanf(url.path,"/%[^/]s",DBName);
+	}
+
 	DBPort=url.port;
 	if((s=strchr(url.auth,':'))){
 		*s=0;
@@ -415,7 +441,7 @@ int res;
 	}else{
 		DBUser=strdup(url.auth);
 	}
-	DBType=UdmStr2DBType(url.schema);
+	
 	return(0);
 }
 
@@ -442,11 +468,12 @@ FILE *config;
 char str[UDMSTRSIZ]="";
 char str1[UDMSTRSIZ]="";
 char str2[UDMSTRSIZ]="";
+char *ch;
 int has_dict=0;
 
 
 	if(config_level==0){ /* Do some initialization */
-		csrv=(UDM_SERVER*)malloc(sizeof(UDM_SERVER));
+		csrv=(UDM_SERVER*)UdmXmalloc(sizeof(UDM_SERVER));
 		InitServer(csrv);
 		UdmSetMaxURLNumber(-1);
 		sprintf(_user_agent,"%s/%s",UDM_USER_AGENT,VERSION);
@@ -610,6 +637,16 @@ int has_dict=0;
 			if(sscanf(str+5,"%s",str1)&&strcmp(str1,""))
 				csrv->proxy=strdup(str1);
 		}else
+		if(!UDM_STRNCASECMP(str,"Category")){
+			UDM_FREE(csrv->category);
+			if(sscanf(str+9,"%s",str1))
+			csrv->category=strdup(str1);
+		}else
+		if(!UDM_STRNCASECMP(str,"Tag")){
+			UDM_FREE(csrv->tag);
+			if(sscanf(str+3,"%s",str1))
+			csrv->tag=strdup(str1);
+		}else
 		if(!UDM_STRNCASECMP(str,"HTTPHeader")){
 			if(sscanf(str+11,"%[^\n\r]s",str1)&&strcmp(str1,"")){
 				if(!UDM_STRNCMP(str1,"User-Agent: ")){
@@ -628,9 +665,18 @@ int has_dict=0;
 #endif
 		if(!UDM_STRNCASECMP(str,"AuthBasic")){
 			UDM_FREE(csrv->basic_auth);
+			UDM_FREE(csrv->user);
+			UDM_FREE(csrv->passwd);
 			if(sscanf(str+9,"%s",str1)){
 				csrv->basic_auth=(char*)malloc(BASE64_LEN(strlen(str1)));
 				base64_encode(str1,csrv->basic_auth,strlen(str1));
+                                if ((ch=strchr(str1, ':'))){
+                            		csrv->user=(char*)UdmXmalloc(ch-str1+1);
+                            		udm_snprintf(csrv->user, ch-str1+1, "%s", str1);
+                            		csrv->passwd=(char*)UdmXmalloc(strlen(ch)+1);
+                            		sprintf(csrv->passwd, "%s", ch+1);
+                                }
+
 			}
 		}else
 		if(!UDM_STRNCASECMP(str,"HTDBList")){
@@ -685,8 +731,7 @@ int has_dict=0;
 		else	IF_BOOL_PARAM(str,"DeleteNoServer",csrv->delete_no_server,1)
 		else	IF_BOOL_PARAM(str,"ForceIISCharset1251",Force1251,0)
 		else	IF_BOOL_PARAM(str,"Clones",csrv->use_clones,1)
-
-		else	IF_NUM_PARAM(str,"Tag",csrv->hint,0)
+		
 		else	IF_NUM_PARAM(str,"MaxNetErrors",csrv->max_net_errors,UDM_MAXNETERRORS)
 		else	IF_NUM_PARAM(str,"ReadTimeOut",csrv->read_timeout,UDM_READ_TIMEOUT)
 		else	IF_NUM_PARAM(str,"Period",csrv->period,UDM_DEFAULT_REINDEX_TIME)
