@@ -26,6 +26,7 @@
 #include "udm_utils.h"
 #include "udm_log.h"
 #include "udm_conf.h"
+#include "udm_crc32.h"
 
 int UdmInitDB(){
 	return(0);
@@ -38,7 +39,7 @@ static char udmver[128];
 }
 
 static char * ul = NULL;
-static int ttag = -1;
+static char * ttag = NULL;
 
 char * escstr(char *x,char *y){
 	return(strcpy(x,y));
@@ -188,16 +189,15 @@ int UdmDBErrorCode(void *db){
 
 
 /*************  Words ****************************/
-extern int UdmStrCRC32(char * buf);
 typedef struct cstruct {
-	int	url_id;
-	int	wrd_id;
-	char	weight;
+	int		url_id;
+	udmcrc32_t	wrd_id;
+	char		weight;
 } UDM_CRCWORD;
 int UdmDeleteWordFromURL(UDM_INDEXER* Indexer,int url_id){
 	return(0);
 }
-int UdmStoreWords(UDM_INDEXER * Indexer, int url_id){
+int UdmStoreWords(UDM_INDEXER * Indexer, int url_id, int status){
 	if (InitDB(((DB*)(Indexer->db)))){
 		UdmLog(Indexer->handle, UDM_LOG_ERROR, "Error: %s", ((DB*)(Indexer->db))->errmsg);
 		exit(1);
@@ -297,16 +297,14 @@ UDM_DOCUMENT * Result;
 			Result->next_index_time=NULL;
 			Result->keywords=NULL;
 			Result->description=NULL;
-			Result->crc=NULL;
-
 			Result->url=strdup(URLs[i].url);
 			Result->url_id=URLs[i].url_id;
 			Result->size=0;
 			Result->status=0;
 			Result->last_index_time=strdup("0");
 			Result->hops=URLs[i].hops;
-			Result->crc=strdup("___");
-			Result->last_modified=strdup("");
+			Result->crc32=0;
+			Result->last_mod_time=0;
 			return(Result);
 		}
 	}
@@ -336,15 +334,16 @@ int UdmUpdateUrl(UDM_INDEXER* Indexer,int url_id,int status,int period){
 #define XVAL(x)		(!x?"-":!x[0]?"-":x)
 int UdmLongUpdateUrl(UDM_INDEXER* Indexer,
 	int url_id,int status,int is_changed,
-	int size,int period,int hint,int index,
-	char *last_modified_escaped,
+	int size,int period,char * tag,int index,
+	time_t last_mod_time,
 	char *text_escaped,
 	char *title_escaped,
 	char *content_type,
 	char *keywords_escaped,
 	char *descript_escaped,
-	char *digest,
-	char *lang
+	udmcrc32_t crc32,
+	char *lang,
+	char *category
 #ifdef NEWS_EXT
 	,
 	char *hd_date,
@@ -356,7 +355,8 @@ int UdmLongUpdateUrl(UDM_INDEXER* Indexer,
 #endif
 	){
 	
-	int i;FILE *f;
+	int i;
+	FILE *f;
 
 	if (InitDB(((DB*)(Indexer->db)))){
 	    UdmLog(Indexer->handle, UDM_LOG_ERROR, "Error: %s", ((DB*)(Indexer->db))->errmsg);
@@ -368,13 +368,14 @@ int UdmLongUpdateUrl(UDM_INDEXER* Indexer,
 	for(i=0;i<n_urls;i++)
 	if(URLs[i].url_id==url_id){
 
-		fprintf(f,"%d\t%d\t%d\t%d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
-			url_id,hint,status,size,
+		fprintf(f,"%d\t%s\t%d\t%d\t%s\t%s\t%u\t%s\t%li\t%s\t%s\t%s\t%s\n",
+			url_id,
+			XVAL(tag),status,size,
 			XVAL(content_type),
 			XVAL(lang),
-			XVAL(digest),
+			crc32,
 			XVAL(URLs[i].url),
-			XVAL(last_modified_escaped),
+			last_mod_time,
 			XVAL(text_escaped),
 			XVAL(title_escaped),
 			XVAL(keywords_escaped),
@@ -387,14 +388,17 @@ int UdmLongUpdateUrl(UDM_INDEXER* Indexer,
 
 
 /************* Clones ***************************/
-int UdmFindOrigin(UDM_INDEXER* Indexer,char *crc){
+int UdmFindOrigin(UDM_INDEXER* Indexer, udmcrc32_t crc32, int size){
 	return(0);
 }
+
 int UdmUpdateClone(UDM_INDEXER* Indexer,int url_id,int status,int period,
-	char *content_type,char *last_modified_escaped,char *digest){
+	char *content_type,time_t last_mod_time,udmcrc32_t crc32){
+
 	return(IND_OK);
 }
-UDM_DOCUMENT * UdmCloneList(void * db,char *crc){
+
+UDM_DOCUMENT * UdmCloneList(void * db, udmcrc32_t crc32){
 	return(NULL);
 }
 int UdmRegisterChild(UDM_INDEXER *Indexer, int parent_id, int child_id){
@@ -471,7 +475,7 @@ static int cmpurlid(const void *s1,const void *s2){
 UDM_DOCUMENT * UdmFind(void * db,char *q,int np,int ps, int search_mode,int sort_order,char *wordinfo,int *found){
 char *w,*lasttok;
 char *words[100];
-int  cwords[100];
+udmcrc32_t cwords[100];
 char str[UDMSTRSIZ];
 char curword[100];
 int weight;
@@ -627,9 +631,9 @@ int skip=0;
 		r[i].url_id=wrd[i].url_id;
 		r[i].rating=wrd[i].count;
 		r[i].content_type=NULL;
-		r[i].crc=NULL;
+		r[i].crc32=0;
 		r[i].url=NULL;
-		r[i].last_modified=NULL;
+		r[i].last_mod_time=0;
 		r[i].text=NULL;
 		r[i].title=NULL;
 		r[i].keywords=NULL;
@@ -642,17 +646,17 @@ int skip=0;
 		url_id=atoi(str);
 		for(i=0;i<nwrd;i++){
 			if(url_id==r[i].url_id){
-				int tag=-1;
+				char * tag=NULL;
 				int matched=1;
 				w=UdmGetToken(str,"\t",&lasttok);
 				for(j=0;w;j++){
 					switch(j){
-						case  1: tag=atoi(w);break;
+						case  1: tag=strdup(w);break;
 						case  3: r[i].size=atoi(w);break;
 						case  4: r[i].content_type=strdup(w);break;
-						case  6: r[i].crc=strdup(w);break;
+						case  6: r[i].crc32=(udmcrc32_t)strtol(w, NULL, 10);break;
 						case  7: r[i].url=strdup(w);break;
-						case  8: r[i].last_modified=strdup(w);break;
+						case  8: r[i].last_mod_time=atol(w);break;
 						case  9: r[i].text=strdup(w);break;
 						case 10: r[i].title=strdup(w);break;
 						case 11: r[i].keywords=strdup(w);break;
@@ -664,14 +668,12 @@ int skip=0;
 				/* Check "ul" subsection match  */
 				if(ul)matched=(strstr(r[i].url,ul)!=NULL);
 				/* Check "tag" subsection match */
-				if((matched)&&(ttag!=-1))matched=(ttag==tag);
+				if((matched)&&(ttag)&&(tag))matched=(!strcmp(ttag,tag));
 
 				if(!matched){
 					/* Skip this URL */
 					UDM_FREE(r[i].content_type);
-					UDM_FREE(r[i].crc);
 					UDM_FREE(r[i].url);
-					UDM_FREE(r[i].last_modified);
 					UDM_FREE(r[i].text);
 					UDM_FREE(r[i].title);
 					UDM_FREE(r[i].keywords);
@@ -714,14 +716,22 @@ int UdmDBImportAffixes(void *db){
     return(0);
 }
 
+UDM_CATEGORY * UdmCatPath(void * db,char * category){
+	return(NULL);
+}
+
+UDM_CATEGORY * UdmCatList(void * db,char * category){
+	return(NULL);
+}
+
 __INDLIB__ int  UdmGetStatistics(){
 	return(IND_OK);
 }
 int UdmMarkForReindex(UDM_INDEXER* Indexer){
 	return(IND_OK);
 }
-__INDLIB__ int UdmAddTagLimit(int tag){
-	ttag=tag;
+__INDLIB__ int UdmAddTagLimit(char * tag){
+	ttag=strdup(tag);
 	return(0);
 }
 __INDLIB__ int UdmAddStatusLimit(int status){
@@ -735,9 +745,14 @@ __INDLIB__ int UdmAddURLLimit(char *URL){
 __INDLIB__ int UdmAddLangLimit(char *URL){
 	return(0);
 }
+
+__INDLIB__ int UdmAddCatLimit(char * cat){
+	return(0);
+}
+
 __INDLIB__ int UdmClearURLLimit(){
-	UDM_FREE(ul)
-	ttag=-1;
+	UDM_FREE(ul);
+	UDM_FREE(ttag);
 	return(0);
 }
 __INDLIB__ int UdmClearLimits(){
@@ -747,5 +762,8 @@ __INDLIB__ int UdmClearLimits(){
 __INDLIB__ int UdmGetReferers(UDM_INDEXER* Indexer){
 	return(IND_OK);
 }
+__INDLIB__ int UdmAddTimeLimit(struct udm_stl_info_t * stl){
+ return 0;
+}
 
 #endif
