@@ -31,14 +31,13 @@
 #include "udm_hrefs.h"
 #include "udm_utils.h"
 #include "udm_mutex.h"
-#include "udm_md5.h"
+#include "udm_crc32.h"
+#include "udm_xmalloc.h"
 
 #ifdef NEWS_EXT
 #include "udm_parsedate.h"
 #endif
 
-#define MD5_DIGEST(data, size, digest) md5_buffer_str(data, size, digest)
-
 static int have_targets=1;
 static char *url_file_name=NULL;
 
@@ -244,6 +243,7 @@ char *title){
 
 	int inbody=0;
 	int inscript=0;
+	int instyle=0;
 	int intitle=0;
 	int hrefonly=0;
 	int comment=0;
@@ -286,56 +286,51 @@ char *title){
 			UdmParseTag(&tag,tmp);
 
 			/* Convert to lower case */
-			for(s=tag.name;*s;*s=tolower(*s),s++);
+			for(s=tag.tag;*s;*s=tolower(*s),s++);
 
-			/* Eliminate whether opening or closing tag */
-			if((opening=(tag.name[0]!='/')))s=tag.name;
-			else				s=tag.name+1;
+			/* Detect whether opening or closing tag */
+			if((opening=(tag.tag[0]!='/')))s=tag.tag;
+			else				s=tag.tag+1;
 
 			/* Let's find tag name in order of frequency */
 
-			if(!strcmp(s,"a"))		href=tag.href;	/*117941*/
+			if(!strcmp(s,"a"))href=tag.href;/*117941*/
 			else	if(!strcmp(s,"title"))	intitle=opening;/*6192*/
-			else	
-			if(!strcmp(s,"body")){/*5146*/
-				if((inbody=opening)){
-					int DCIndex;
-#ifdef USE_CHARSET_GUESSER
-					DCIndex = UdmGuessCharset(content, UdmGetCharset(CurSrv->charset));
-#else
-					if(Indexer->charset){
-						DCIndex=UdmGetCharset(Indexer->charset);
-					}else{
-						DCIndex=UdmGetCharset(CurSrv->charset);
-					}
-#endif
-					UdmRecode(content, DCIndex,Indexer->local_charset);
-					UdmRecode(title,   DCIndex,Indexer->local_charset);
-					UdmRecode(descript,DCIndex,Indexer->local_charset);
-					UdmRecode(keywords,DCIndex,Indexer->local_charset);
-				}
-			}else
-			if((!strcmp(tag.name,"meta"))&&(tag.equiv)&&(tag.content)){
-			
-				if((!strcasecmp(tag.equiv,"Content-Type"))&&(!Indexer->charset)){
+			else	if(!strcmp(s,"body"))	inbody=opening;	/*5146*/
+			else
+			if((!strcmp(tag.tag,"meta"))&&(tag.name)&&(tag.content)){
+#ifndef USE_CHARSET_GUESSER	/* meta 4160 */
+				if((!strcasecmp(tag.name,"Content-Type"))&&(!Indexer->charset)){
 					char *p;
 					if((p=strstr(tag.content,"charset="))){
 						Indexer->charset = p + 8;
+						UdmRecode(content,UdmGetCharset(Indexer->charset),Indexer->local_charset);
 					}
 				}else
-				if(!strcasecmp(tag.equiv,"refresh")){
+#endif
+				if(!strcasecmp(tag.name,"refresh")){
 					if((href=strstr(tag.content,"URL=")))
 						href+=4;
 				}else
-				if(!hrefonly&&!strcasecmp(tag.equiv,"keywords")){
+				if(!hrefonly&&!strcasecmp(tag.name,"keywords")){
 					strncpy(keywords,tag.content,UDM_MAXKEYWORDSIZE-1);
 					keywords[UDM_MAXKEYWORDSIZE-1]=0;
+					if(index&&CurSrv->keywordweight){
+						strcpy(str,keywords);
+						ParseText(Indexer,CurSrv,str,
+							CurSrv->keywordweight,1);
+					}
 				}else
-				if(!hrefonly&&!strcasecmp(tag.equiv,"description")){
+				if(!hrefonly&&!strcasecmp(tag.name,"description")){
 					strncpy(descript,tag.content,UDM_MAXDESCSIZE-1);
 					descript[UDM_MAXDESCSIZE-1]=0;
+					if(index&&CurSrv->descweight){
+						strcpy(str,descript);
+						ParseText(Indexer,CurSrv,str,
+							CurSrv->descweight,1);
+					}
 				}else
-				if(!strcasecmp(tag.equiv,"robots")&&CurSrv->userobots&&tag.content){
+				if(!strcasecmp(tag.name,"robots")&&CurSrv->userobots&&tag.content){
 					char * lt;
 					char * s;
 					s=UdmGetWord(tag.content,&lt,Indexer->local_charset);
@@ -361,10 +356,11 @@ char *title){
 					}
 				}
 			}
-			else	if(!strcmp(s,"img"))	href=tag.src;	/*2786*/
-			else	if(!strcmp(s,"link"))	href=tag.href;	/*2241*/
+			else	if(!strcmp(s,"img"))	href=tag.src;/*2786*/
+			else	if(!strcmp(s,"link"))	href=tag.href;/*2241*/
 			else	if(!strcmp(s,"frame"))	href=tag.src;
 			else	if(!strcmp(s,"script"))	inscript=opening;
+			else	if(!strcmp(s,"style"))	instyle=opening;
 			else	if(!strcmp(s,"area"))	href=tag.href;
 			else	if((!strcmp(s,"base"))&&(tag.href)){
 				/* <BASE HREF="xxx"> stuff            */
@@ -378,10 +374,12 @@ char *title){
 				/* if delete_no_server is unset         */
 				/* This is  actually dirty hack. We     */
 				/* must check that hostname is the same */
-
+				
+				
 				if((CurSrv->delete_no_server)||(CurSrv->outside)){
 					UDM_URL baseURL;
 					int res;
+					
 					if(!(res=UdmParseURL(&baseURL,tag.href))){
 						UdmParseURL(curURL,tag.href);
 					}else{
@@ -554,7 +552,7 @@ char *title){
 					break;
 				}
 			}
-			if((!comment&&inbody&&!inscript)&&((len=strlen(text))<UDM_MAXTEXTSIZE-2)){
+			if((!comment&&inbody&&!inscript&&!instyle)&&((len=strlen(text))<UDM_MAXTEXTSIZE-2)){
 				len=UDM_MAXTEXTSIZE-2-len;
 				if(*text)strcat(text," ");
 				strncat(text,tmp,len);
@@ -566,26 +564,16 @@ char *title){
 				strncat(title,tmp,len);
 				title[UDM_MAXTITLESIZE-1]=0;
 			}
-			if(!comment&&!hrefonly&&index&&!inscript){
-				ParseText(Indexer,CurSrv,tmp,CurSrv->bodyweight*inbody,1);
+			if(!comment&&!hrefonly&&index&&!inscript&&!instyle){
+				ParseText(Indexer,CurSrv,tmp,
+					CurSrv->bodyweight*inbody+
+					CurSrv->titleweight*intitle,1);
 			}
 			free(tmp);
 			htok=hend;
 			break;
 		}
 	}
-	if(index&&CurSrv->keywordweight){
-		strcpy(str,title);
-		ParseText(Indexer,CurSrv,str,CurSrv->titleweight,1);
-	}
-	if(index&&CurSrv->keywordweight){
-		strcpy(str,keywords);
-		ParseText(Indexer,CurSrv,str,CurSrv->keywordweight,1);
-	}
-	if(index&&CurSrv->descweight){
-		strcpy(str,descript);
-		ParseText(Indexer,CurSrv,str,CurSrv->descweight,1);
-	}
 	return(0);
 }
 
@@ -599,14 +587,12 @@ static void FreeDoc(UDM_DOCUMENT *Result){
 	if(!Result)return;
 	UDM_FREE(Result->url);
 	UDM_FREE(Result->content_type);
-	UDM_FREE(Result->last_modified);
 	UDM_FREE(Result->title);
 	UDM_FREE(Result->text);
 	UDM_FREE(Result->last_index_time);
 	UDM_FREE(Result->next_index_time);
 	UDM_FREE(Result->keywords);
 	UDM_FREE(Result->description);
-	UDM_FREE(Result->crc);
 	free(Result);
 }
 
@@ -743,7 +729,7 @@ static int StartUp(UDM_INDEXER * Indexer, int index_flags){
 __INDLIB__ int UdmIndexNextURL(UDM_INDEXER *Indexer,int index_flags){
 char request[UDMSTRSIZ]="";
 char reason[UDMSTRSIZ]="";
-char digest[33];
+udmcrc32_t crc32;
 
 char *lt,*tok;
 char *content=NULL,*header,*content_type,*location,*statusline;
@@ -757,9 +743,9 @@ int result=IND_UNKNOWN;
 
 UDM_URL		curURL;
 UDM_URL		realURL;
-UDM_DOCUMENT	* Doc=NULL;
-UDM_SERVER	* CurSrv=0;
-UDM_ALIAS	* Alias;
+UDM_DOCUMENT	*Doc=NULL;
+UDM_SERVER	*CurSrv=0;
+UDM_ALIAS	*Alias;
 
 char text[2*UDM_MAXTEXTSIZE]="";
 char text_escaped[2*UDM_MAXTEXTSIZE+1]="";
@@ -767,13 +753,13 @@ char keywords[UDM_MAXKEYWORDSIZE]="";
 char keywords_escaped[2*UDM_MAXKEYWORDSIZE+1]="";
 char descript[UDM_MAXDESCSIZE]="";
 char descript_escaped[2*UDM_MAXDESCSIZE+1]="";
-char last_modified[32]="";
-char last_modified_escaped[32*2+1]="";
+time_t last_mod_time=0;
+char buf[UDM_MAXTIMESTRLEN]="";
 char title[UDM_MAXTITLESIZE]="";	
 char title_escaped[UDM_MAXTITLESIZE*2+1];
-char content_type_escaped[256]="";
 char subj[UDM_MAXTITLESIZE]="";
 char from[UDM_MAXKEYWORDSIZE]="";
+char content_type_escaped[256]="";
 
 #ifdef NEWS_EXT
 /* I need to retain newsgroup-info, so I introduce new variables */
@@ -821,7 +807,8 @@ int mimeno;
 	}else
 	if(!have_targets){
 		result=IND_NO_TARGET;
-	}if((CurURLNumber>=MaxURLNumber)&&(MaxURLNumber>=0)){
+	}else
+	if((CurURLNumber>=MaxURLNumber)&&(MaxURLNumber>=0)){
 		result=IND_NO_TARGET;
 	}else{
 		/* Get Next URL to be indexed from the database */
@@ -848,7 +835,6 @@ int mimeno;
 	/* It allows terminate indexing faster                 */
 	/* We include this line in several places of this func */
 	if(Indexer->action==IND_TERMINATED)result=IND_TERMINATED;
-
 	if(result)return(result);
 
 	/* Alloc buffer for document */
@@ -921,11 +907,11 @@ int mimeno;
 		result=UdmDeleteUrl(Indexer,Doc->url_id);
 	}else
 	/* Check whether FTP is used without proxy */
-	if((!strcmp(curURL.schema,"ftp"))&&(!CurSrv->proxy)){
+/*	if((!strcmp(curURL.schema,"ftp"))&&(!CurSrv->proxy)){
 		UdmLog(Indexer->handle,UDM_LOG_WARN,"FTP is available via proxy only, skipped");
 		result=UdmUpdateUrl(Indexer,Doc->url_id,504,CurSrv->period);
 	}else
-	/* Check that too many errors on this server */
+*/	/* Check for too many errors on this server */
 	if(CurSrv->net_errors>CurSrv->max_net_errors){
 		UdmLog(Indexer->handle,UDM_LOG_WARN,"Too many network errors for this server, skipped");
 		result=UdmUpdateUrl(Indexer,Doc->url_id,504,CurSrv->period);
@@ -957,9 +943,11 @@ int mimeno;
 		CurSrv->proxy?"":realURL.filename);
 
 	/* Add If-Modified-Since header */
-	if((Doc->status)&&(Doc->last_modified[0])&&(!reindex))
+	if((Doc->status)&&(Doc->last_mod_time)&&(!reindex)){
+		UdmTime_t2HttpStr(Doc->last_mod_time, buf);
 		sprintf(UDM_STREND(request),"If-Modified-Since: %s\r\n",
-			Doc->last_modified);
+			buf);
+	}
 
 	/* HTTP and FTP specific stuff */
 	if((!strcmp(realURL.schema,"http"))||(!strcmp(realURL.schema,"ftp"))){
@@ -1053,7 +1041,7 @@ int mimeno;
 				UdmMaxDocSize(),
 				CurSrv->read_timeout);
 		}else
-		if((!strcmp(realURL.schema,"ftp"))){
+		if((!strcmp(realURL.schema,"ftp"))&&(CurSrv->proxy)){
 			size = UdmHTTPGet(
 				CurSrv->proxy?CurSrv->proxy:realURL.hostname,
 				CurSrv->proxy?CurSrv->proxy_port:(realURL.port?realURL.port:80),
@@ -1061,6 +1049,14 @@ int mimeno;
 				UdmMaxDocSize(),
 				CurSrv->read_timeout);
 		}
+#endif
+#ifdef USE_FTP
+		if ((!strcmp(curURL.schema,"ftp"))&&(!CurSrv->proxy)){
+		    size = UdmFTPGet(realURL.hostname,realURL.port?realURL.port:21,
+			curURL.path, curURL.filename[0]=='\0'?NULL:curURL.filename, CurSrv->user,
+			CurSrv->passwd, Indexer, UdmMaxDocSize(),
+			CurSrv->read_timeout, Doc->last_mod_time, (Method==UDM_HEAD?1:0));
+		}
 #endif
 	}
 
@@ -1131,16 +1127,16 @@ int mimeno;
 	realsize=size;
 	status=changed=1;origin=0;
 	content_type=location=Indexer->charset=statusline=NULL;
-	digest[0]=0;text[0]=0;title[0]=0;keywords[0]=0;
-	descript[0]=0;subj[0]=0;last_modified[0]=0;from[0]=0;
-	strcpy(last_modified,Doc->last_modified?Doc->last_modified:"");
+	crc32=0;text[0]=0;title[0]=0;keywords[0]=0;
+	descript[0]=0;subj[0]=0;last_mod_time=0;from[0]=0;
+	last_mod_time=Doc->last_mod_time;
 	
 
 	index	=CurSrv->gindex;
 	follow	=CurSrv->gfollow;
 
 	size-=(content-Indexer->buf);     /* Could be modified by Content-Length    */
-	realsize-=(content-Indexer->buf); /* Will be safe for DIGEST, Parsers etc   */
+	realsize-=(content-Indexer->buf); /* Will be safe for crc32, Parsers etc   */
 
 	UDM_THREADINFO(Indexer->handle,"Parsing",Doc->url);
 	/* Now lets parse response header lines */
@@ -1163,6 +1159,10 @@ int mimeno;
 			content_type=tok+14;
 			if((p=strstr(content_type,"charset=")))
 				Indexer->charset = p + 8;
+			/* Store content_type right now  */
+			/* It can be modified  after     */
+			/* possible external parser call */
+			escstr(content_type_escaped,content_type);
 		}else
 		if(!UDM_STRNCASECMP(tok,"Location: ")){
 			location=tok+10;
@@ -1198,9 +1198,7 @@ int mimeno;
 		}else
 #endif
 		if(!UDM_STRNCASECMP(tok,"Last-Modified: ")){
-			UDM_STRNCPY(last_modified,tok+15);
-			
-			escstr(last_modified_escaped,last_modified);
+			last_mod_time=UdmHttpDate2Time_t(tok+15);
 		}
 		UdmLog(Indexer->handle,UDM_LOG_DEBUG,"%s",tok);
 		tok=UdmGetToken(NULL,"\r\n",&lt);
@@ -1376,10 +1374,12 @@ int mimeno;
 			UdmRecode(content,DCindex,Indexer->local_charset);
 		}
 		if(Method!=UDM_HEAD){
-			MD5_DIGEST(content,realsize,digest);
-			changed=strcmp(digest,Doc->crc);
+		        /* MD5_DIGEST(content,realsize,digest); */
+			/* changed=strcmp(digest,Doc->crc); */
+			crc32=UdmCRC32(content, realsize);
+			changed=!(crc32==Doc->crc32);
 			if(CurSrv->use_clones){
-				origin=UdmFindOrigin(Indexer,digest);
+				origin=UdmFindOrigin(Indexer, crc32, size);
 				origin=((origin==Doc->url_id)?0:origin);
 			}
 		}
@@ -1393,11 +1393,26 @@ int mimeno;
 	/* HTML text */
 	if(!UDM_STRNCASECMP(content_type,"text/html")){
 
+		if (Indexer->charset) {
+			UdmRecode(content,UdmGetCharset(Indexer->charset),Indexer->local_charset);
+		} else {
+			int DCindex=0;
+#ifdef USE_CHARSET_GUESSER
+			DCindex = UdmGuessCharset(content, UdmGetCharset(CurSrv->charset));
+#else
+			DCindex = UdmGetCharset(CurSrv->charset);
+#endif
+			UdmRecode(content,DCindex,Indexer->local_charset);
+		}
 		if(Method!=UDM_HEAD){
+			/*
 			MD5_DIGEST(content,realsize,digest);
 			changed=strcmp(digest,Doc->crc);
+			*/
+			crc32=UdmCRC32(content, realsize);
+			changed=!(crc32==Doc->crc32);
 			if(CurSrv->use_clones){
-				origin=UdmFindOrigin(Indexer,digest);
+				origin=UdmFindOrigin(Indexer, crc32, size);
 				origin=((origin==Doc->url_id)?0:origin);
 			}
 		}
@@ -1410,10 +1425,15 @@ int mimeno;
 		/* Unknown Content-Type */
 		index=0; /* What this line for? */
 		if(Method!=UDM_HEAD){
+			/*
 			MD5_DIGEST(content,realsize,digest);
 			changed=strcmp(digest,Doc->crc);
+			*/
+			crc32=UdmCRC32(content, realsize);
+			changed=!(crc32==Doc->crc32);
+
 			if(CurSrv->use_clones){
-				origin=UdmFindOrigin(Indexer,digest);
+				origin=UdmFindOrigin(Indexer, crc32, size);
 				origin=((origin==Doc->url_id)?0:origin);
 			}
 		}
@@ -1446,7 +1466,7 @@ int mimeno;
 		result=UdmDeleteWordFromURL(Indexer,Doc->url_id);
 		if(result==IND_OK)
 			result=UdmUpdateClone(Indexer,Doc->url_id,status,CurSrv->period,
-				content_type,last_modified_escaped,digest);
+				content_type_escaped,last_mod_time,crc32);
 	}else
 	if((!changed)&&(!reindex)){
 		result=UdmUpdateUrl(Indexer,Doc->url_id,status,CurSrv->period);
@@ -1456,7 +1476,7 @@ int mimeno;
 			strcpy(str,subj);
 			ParseText(Indexer,CurSrv,str,CurSrv->titleweight,1);
 		}
-		if((result=UdmStoreWords(Indexer,Doc->url_id))==IND_OK){
+		if((result=UdmStoreWords(Indexer,Doc->url_id, Doc->status))==IND_OK){
 			if(LockProc)LockProc(UDM_LOCK,UDM_LOCK_TARGET);
 			if(UdmStoreHrefs(Indexer))
 				have_targets=1;
@@ -1493,7 +1513,8 @@ int mimeno;
 			s=title;while(*s){if(strchr("\t\n\r",*s))*s=' ';s++;}
 			title[UDM_MAXTITLESIZE-1]=0;
 			escstr(title_escaped,title);
-			escstr(content_type_escaped,content_type);
+			
+			
 
 			/* Guess the language */
 			qsort((void*)(Indexer->lang+1),UDM_LANGPERDOC-1,sizeof(UDM_LANG),cmplang);
@@ -1560,10 +1581,12 @@ int mimeno;
 
 			result=UdmLongUpdateUrl(Indexer,Doc->url_id,status,
 				changed,size,CurSrv->period,
-				CurSrv->hint,index,last_modified_escaped,
+				CurSrv->tag?CurSrv->tag:"",
+				index,last_mod_time,
 				text_escaped,title_escaped,
 				content_type_escaped,keywords_escaped,
-				descript_escaped,digest,lang,
+				descript_escaped,crc32,lang,
+				CurSrv->category?CurSrv->category:"",
 				HeaderDateEsc,
 				HeaderSubjectEsc,
 				HeaderFromEsc,
@@ -1572,10 +1595,12 @@ int mimeno;
 				MessageIdEsc);
 #else
 			result=UdmLongUpdateUrl(Indexer,Doc->url_id,status,changed,size,CurSrv->period,
-				CurSrv->hint,index,last_modified_escaped,
+				CurSrv->tag?CurSrv->tag:"",
+				index,last_mod_time,
 				text_escaped,title_escaped,
 				content_type_escaped,keywords_escaped,
-				descript_escaped,digest,lang);
+				descript_escaped,crc32,lang,
+				CurSrv->category?CurSrv->category:"");
 #endif			
 		}
 	}
@@ -1603,6 +1628,11 @@ UDM_INDEXER * result;
 	result->charset=NULL;
 	result->local_charset=UdmGetDefaultCharset();
 	result->db=UdmAllocDB(UDM_OPEN_MODE_WRITE);
+        /* Reserve connection structure */
+        result->connp = (UDM_CONN*)UdmXmalloc(sizeof(UDM_CONN));
+        result->connp->indexer = result;
+        result->connp->connp = (UDM_CONN*)UdmXmalloc(sizeof(UDM_CONN));
+        result->connp->connp->indexer = result;
 	return(result);
 }
 __INDLIB__ void UdmFreeIndexer(UDM_INDEXER *Indexer){
