@@ -33,6 +33,9 @@
 #include "udm_proto.h"
 #include "udm_utils.h"
 #include "udm_mutex.h"
+#include "udm_socket.h"
+#include "udm_ftp.h"
+#include "udm_xmalloc.h"
 
 #ifndef INADDR_NONE
 #define INADDR_NONE (unsigned long)(-1)
@@ -40,10 +43,9 @@
 
 char *MirrorRoot = NULL;
 char *MirrorHeadersRoot = NULL;
-static int NET_BUF_SIZE=10240;
 
 
-/***************  AddType stuff **********************/
+/********* AddType stuff *********************/
 static int mmimes=0;
 static int nmimes=0;
 
@@ -53,6 +55,7 @@ typedef struct mime_struct {
 } MIME;
 static MIME * Mime=NULL;
 
+
 int UdmAddType(char * mime_type,char *reg, char *errstr){
 #define ERRSTRSIZE 100
 char regerrstr[ERRSTRSIZE];
@@ -86,6 +89,7 @@ int i;
 	nmimes=mmimes=0;
 	UDM_FREE(Mime);
 }
+
 static char * ContentType(char *url){
 int i;
 #define NS 10
@@ -98,9 +102,8 @@ int err;
 	return(NULL);
 }
 
+/**************************************************/
 
-
-/************************* Mirroring stuff ********************/
 int UdmBuild(char *path, int omode) {
 	struct stat sb;
 	mode_t numask, oumask;
@@ -176,6 +179,8 @@ int UdmBuild(char *path, int omode) {
 }
 
 
+/*************** Mirroring stuff ************************/
+
 int   UdmMirrorGET(char *schema, char *hostname, char *path,
 		char *filename, char *buf, int maxsize,
 		int days, char *errstr) {
@@ -302,8 +307,7 @@ int   UdmMirrorPUT(char *schema, char *hostname, char *path,
 }
 
 
-
-/************** WinSock initialization ***********************/
+/************ WinSock Initialization *************/
 
 #if (WIN32|WINNT)
 static int isWSA = 0;
@@ -330,8 +334,8 @@ static void stopWSA(){
 #endif
 
 
+/************** Open TCP Connect **************/
 
-/****************** Open TCP Connect **********************/
 static int open_host(char *hostname,int port, int timeout)
 {
 int net;
@@ -353,11 +357,7 @@ struct sockaddr_in sin;
 	if ((sin.sin_addr.s_addr=inet_addr(hostname)) != INADDR_NONE){
 		sin.sin_family=AF_INET;
 	}else{
-	
-		if(LockProc)LockProc(UDM_LOCK,UDM_LOCK_RESOLVE);
-		/* It is not reentrant, hide this under mutex */
 		host=gethostbyname(hostname);
-		if(LockProc)LockProc(UDM_UNLOCK,UDM_LOCK_RESOLVE);
 
 		if (host){
 			sin.sin_family=host->h_addrtype;
@@ -376,8 +376,7 @@ struct sockaddr_in sin;
 }
 
 
-
-/***************** HTTP codes and messages *************************/
+/**************** HTTP codes and messages ************/
 
 char *UdmHTTPErrMsg(int code){
 	switch(code){
@@ -460,8 +459,7 @@ int UdmHTTPResponseType(int status){
 }
 
 
-/***********************************************************/
-
+/*******************************************************/
 
 #ifdef USE_HTTP
 int UdmHTTPGet(char *host,int port,char *header,
@@ -520,7 +518,108 @@ int UdmHTTPGet(char *host,int port,char *header,
     return(nread);
 }
 #endif
-  
+
+#ifdef USE_FTP
+int UdmFTPGet(char *host,int port, char *path, char *filename, char *user, char *passwd,
+    UDM_INDEXER *Indexer, int maxsize, int read_timeout, int last_mod_time, int checkonly)
+{
+	int size=0, last_mod_tm, buf_size, code, len;
+	char *con_type, buf[256], *full_path=NULL;
+	
+	/* Check for changing host */
+	if (!Indexer->connp->hostname || 
+	    strcmp(Indexer->connp->hostname, host) || 
+	    (Indexer->connp->connected == NET_NOTCONNECTED)){
+		code=ftp_connect(Indexer->connp, host, port, user, passwd, read_timeout);
+		if (code == -1){
+			if (Indexer->connp->err >0){
+				sprintf(Indexer->buf, "HTTP/1.1 401 OK\r\n\r\n  ");
+				size = strlen(Indexer->buf);
+			}else{
+				size = Indexer->connp->err;
+			}
+		}
+	}
+	
+	if (Indexer->connp->connected == NET_CONNECTED){
+		/* Make listing */
+		if (!filename){
+			code = ftp_cwd(Indexer->connp, path);
+			if (code != -1){
+				code = ftp_list(Indexer->connp, Indexer->connp->connp, path, NULL);
+				if (code == -1){
+					if (Indexer->connp->err >0){
+						sprintf(Indexer->buf, "HTTP/1.1 403 OK\r\n\r\n");
+						size = strlen(Indexer->buf);
+					}else{
+						size = Indexer->connp->err;
+					}
+				}else{
+					udm_snprintf(Indexer->buf, maxsize, "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\n\r\n<html><body>%s</body></html>", 
+						Indexer->connp->connp->buf);
+					size = strlen(Indexer->buf);
+				}
+			}else{
+				if (Indexer->connp->err >0){
+					sprintf(Indexer->buf, "HTTP/1.1 403 OK\r\n\r\n");
+					size = strlen(Indexer->buf);
+				}else{
+					size = Indexer->connp->err;
+				}
+			}
+		}else{
+			con_type = ContentType(filename);
+			len = strlen(path)+strlen(filename);
+			full_path = UdmXmalloc(len+1);
+			udm_snprintf(full_path, len+1, "%s%s", path, filename);
+			last_mod_tm = ftp_mdtm(Indexer->connp,full_path);
+			if(last_mod_tm == -1 && Indexer->connp->err){
+				if (Indexer->connp->err >0){
+					sprintf(Indexer->buf, "HTTP/1.1 404 OK\r\n\r\n");
+					size = strlen(Indexer->buf);
+				}else
+					size = Indexer->connp->err;
+
+			}else if (last_mod_tm == last_mod_time){
+				sprintf(Indexer->buf, "HTTP/1.1 304 OK\r\n\r\n");
+				size = strlen(Indexer->buf);
+			}else{
+				UdmTime_t2HttpStr(last_mod_tm, buf);
+				if (!checkonly){
+					if (!ftp_get(Indexer->connp, Indexer->connp->connp, full_path)){
+						udm_snprintf(Indexer->buf, maxsize, "HTTP/1.1 200 OK\r\nContent-Type: %s\n\rLast-Modified: %s\r\n\r\n", 
+							con_type,buf);
+						size = strlen(Indexer->buf);
+						if (Indexer->connp->connp->buf_len+size >=maxsize)
+							buf_size = maxsize-size;
+						else
+							buf_size = Indexer->connp->connp->buf_len;
+						
+						memcpy(Indexer->buf+size, Indexer->connp->connp->buf, buf_size);
+						size +=buf_size; 
+					}else{
+						if (Indexer->connp->err >0){
+							sprintf(Indexer->buf, "HTTP/1.1 403 OK\r\n\r\n");
+							size = strlen(Indexer->buf);
+						}else{
+							size = Indexer->connp->err;
+			
+						}
+					}
+				}else{
+				    	sprintf(Indexer->buf, "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\nLast-Modified: %s\r\n\r\n", buf);
+					size = strlen(Indexer->buf);
+				}
+			}
+		}
+	}else
+
+	UDM_FREE(full_path);
+	UDM_FREE(Indexer->connp->buf);
+	UDM_FREE(Indexer->connp->connp->buf);
+	return size;
+}
+#endif  
 
 #ifdef USE_NEWS
 int UdmNEWSGet(char *host,int port,char *header,
@@ -761,9 +860,7 @@ char proto[32]="";
 char *content_type=NULL,*s;
 
 	sscanf(header,"%s%s%s",command,filename,proto);
-	strcpy(newfilename,filename);
-	UdmUnescapeCGIQuery(openname,newfilename);
-
+	strcpy(openname,filename);
 	s=openname;
 #if (WIN32|WINNT)
 	while(*s){
@@ -795,8 +892,6 @@ char *content_type=NULL,*s;
 		if((dir=opendir(openname))){
 			strcpy(dist,"HTTP/1.0 200 OK\r\nContent-type: text/html\r\n\r\n<HTML><BODY>\n");
 			while((rec = readdir (dir))){
-				char escaped_name[UDMSTRSIZ]="";
-				char *s,*e;
 				/*
 				This does not work on Solaris
 				is_dir=(rec->d_type==DT_DIR);
@@ -806,21 +901,9 @@ char *content_type=NULL,*s;
 					/* FIXME!!! Handler should be here */
 				}
 				is_dir=((sb1.st_mode&S_IFDIR)>0);
-
-				e=escaped_name;
-				for(s=rec->d_name;*s;s++){
-					if(strchr(" %&<>+[](){}/?#'\"\\;,",*s)){
-						sprintf(e,"%%%X",(int)*s);
-						e+=3;
-					}else{
-						*e=*s;
-						e++;
-					}
-				}
-				*e=0;
 				sprintf(UDM_STREND(dist),"<A HREF=\"%s%s\">%s%s</A>\n",
-					escaped_name,is_dir?"/":"",
-					escaped_name,is_dir?"/":"");
+					rec->d_name,is_dir?"/":"",
+					rec->d_name,is_dir?"/":"");
 			}
 			closedir(dir);
 			strcpy(UDM_STREND(dist),"</BODY><HTML>\n");
